/* ** clientlist.c ******************************* */
/* Functions for control list of clients. */
/* *********************************************** */

/* -- Include File ------------------------------- */
#include <config.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include "clientlist.h"
#include "protocol.h"
#include "tuitated.h"
/* -- Global Data -------------------------------- */
/* -- Prototype Declerations  -------------------- */

/* ** Func AddClient ***************************** */
/*
 * Add Client which has given socket, to Client List.
 * return 0... Success.
 * return 1... Fail.
 */
int AddClient( int Socket )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *cur;
    CLIENTDATA *prev;
/* -- Operation ---------------------------------- */
    cur = ClientList;
    prev = NULL;
    while( cur != NULL ) {
        prev = cur;
        cur = cur->Next;
    }
    cur = (CLIENTDATA *)malloc( sizeof( CLIENTDATA ) );
    if( cur == NULL ) {
        Err( "malloc." );
        return 1;
    }
    cur->soc = Socket;
    cur->mode = MODE_NEUTRAL;
    cur->Selecting = NULL;
    cur->Selected = NULL;
    cur->name = NULL;
    cur->mailaddress = NULL;
    cur->ReceiveMsgBuf[0] = '\0';
    cur->SendMsgList = NULL;
    cur->ShouldBeRemoved = FALSE;
    cur->Next = NULL;
    if( prev != NULL )
        prev->Next = cur;
    else
        ClientList = cur;
    AccessLog( "new client %d", cur->soc );
    return 0;
} /* AddClient */

/* ** Func RemoveClient ************************** */
/*
 * Remove given Client Data pack.
 * If message is still saved in the queue,
 * pick up the flag.
 * return 0... Success.
 * return 1... Fail.
 */
int RemoveClient( CLIENTDATA *p )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *cur;
    CLIENTDATA *prev;
/* -- Operation ---------------------------------- */
    if( p->SendMsgList != NULL ) {
        p->ShouldBeRemoved = TRUE;
        return 0;
    }
    cur = ClientList;
    prev = NULL;
    while( 1 ) {
        if( cur == NULL ) {
            return 1;
        } else if( cur == p ) { /* Found ! */
            /* free members of client */
            close( cur->soc );
            if( cur->name != NULL ) free( cur->name );
            if( cur->mailaddress != NULL ) free( cur->mailaddress );
            /* free client itself */
            if( prev == NULL ) { /* cur == ClientList */
                ClientList = cur->Next;
                free( cur );
            } else {
                prev->Next = cur->Next;
                free( cur );
            }
            break;
        }
        prev = cur;
        cur = cur->Next;
    }
    AccessLog( "remove client %d", cur->soc );
    return 0;
} /* RemoveClient */

/* ** Func DeleteClient ************************** */
/*
 * Force given Client to be removed.
 * return 0... Success.
 * return 1... Fail.
 */
int DeleteClient( CLIENTDATA *p )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *cur;
    CLIENTDATA *prev;
    SENDMSGDATA *curdata;
    SENDMSGDATA *prevdata;
/* -- Operation ---------------------------------- */
    cur = ClientList;
    prev = NULL;
    while( 1 ) {
        if( cur == NULL ) {
            return 1;
        } else if( cur == p ) { /* Found ! */
            /* free members of client */
            close( cur->soc );
            if( cur->name != NULL ) free( cur->name );
            if( cur->mailaddress != NULL ) free( cur->mailaddress );
            curdata = cur->SendMsgList;
            prevdata = NULL;
            while( curdata != NULL ) {
                prevdata = curdata;
                curdata = curdata->Next;
                free( prevdata->data );
                free( prevdata );
            }
            /* free client itself */
            if( prev == NULL ) { /* cur == ClientList */
                ClientList = cur->Next;
                free( cur );
            } else {
                prev->Next = cur->Next;
                free( cur );
            }
            break;
        }
        prev = cur;
        cur = cur->Next;
    }
    AccessLog( "remove client %d", cur->soc );
    return 0;
} /* DeleteClient */

/* ** Func CleanClient *************************** */
/*
 * Remove All Client Data pack.
 * return 0... Success.
 * return 1... Fail.
 */
int CleanClient( void )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *cur;
    CLIENTDATA *prev;
/* -- Operation ---------------------------------- */
    cur = ClientList;
    prev = NULL;
    while( cur != NULL ) {
        prev = cur;
        cur = cur->Next;
        free( prev );
    }
    return 0;
} /* CleanClient */

/* ** Func SetSocketDiscriptorToWrite ********************* */
/*
 * Set socket discriptors of clients which message to be sent.
 * return 0... Success.
 * return 1... Fail.
 */
int SetSocketDiscriptorToWrite( fd_set *FdSet )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *p;
/* -- Operation ---------------------------------- */
    p = ClientList;
    while( p != NULL ) {
        if( p->SendMsgList != NULL )
            FD_SET( p->soc, FdSet );
        p = p->Next;
    }
    return 0;
} /* SetSocketDiscriptorToWrite */

/* ** Func SetSocketDiscriptorToRead ********************* */
/*
 * Set all socket discriptors of current clients.
 * return 0... Success.
 * return 1... Fail.
 */
int SetSocketDiscriptorToRead( fd_set *FdSet )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *p;
/* -- Operation ---------------------------------- */
    p = ClientList;
    while( p != NULL ) {
        if( !p->ShouldBeRemoved )
            FD_SET( p->soc, FdSet );
        p = p->Next;
    }
    return 0;
} /* SetSocketDiscriptorToRead */

/* ** Func SocketDiscriptorNum ********************* */
/*
 * Return max of socket discriptors.
 * return 0... No discrotor. (No client.)
 */
int SocketDiscriptorNum( void )
{
/* -- Local Data --------------------------------- */
    int ret;
    CLIENTDATA *p;
/* -- Operation ---------------------------------- */
    ret = 0;
    p = ClientList;
    while( p != NULL ) {
        if( ret < p->soc ) ret = p->soc;
        p = p->Next;
    }
    return ret;
} /* SocketDiscriptorNum */

/* ** Func MatchSocketDiscriptor ***************** */
/*
 * Return the pointer of CLIENT DATA structure,
 * which was in the given discrptor set.
 * Return NULL if an error occure( not found ).
 */
CLIENTDATA *MatchSocketDiscriptor( const fd_set *FdSet )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *p;
/* -- Operation ---------------------------------- */
    p = ClientList;
    while( p != NULL ) {
        if( FD_ISSET( p->soc, FdSet ) ) {
            return p;
        }
        p = p->Next;
    }
    return NULL;
} /* MatchSocketDiscriptor */

/* ** Func MatchSocket *************************** */
/*
 * Return the pointer of CLIENT DATA structure,
 * which has the given socket discrptor (client ID).
 * Return NULL if an error occure( not found ).
 */
CLIENTDATA *MatchSocket( int soc )
{
/* -- Local Data --------------------------------- */
    CLIENTDATA *p;
/* -- Operation ---------------------------------- */
    p = ClientList;
    while( p != NULL ) {
        if( p->soc == soc ) {
            return p;
        }
        p = p->Next;
    }
    return NULL;
} /* MatchSocket */

/* ** Func ChallengerNum ************************ */
/*
 * Return the number of client which is in
 * SELECTENEMYMODE.
 */
int ChallengerNum( CLIENTDATA *me )
{
/* -- Local Data --------------------------------- */
    int ret = 0;
    CLIENTDATA *p;
/* -- Operation ---------------------------------- */
    p = ClientList;
    while( p != NULL ) {
        if( p->mode == MODE_SELECTENEMY && me != p ) {
            ret++;
        }
        p = p->Next;
    }
    return ret;
} /* ChallengerNum */

